# -*- coding: utf-8 -*-
import base64
import random
import threading
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer

from core.libs import *


class MyHTTPServer(HTTPServer):
    daemon_threads = True

    def process_request_thread(self, request, client_address):
        try:
            self.finish_request(request, client_address)
            self.shutdown_request(request)
        except Exception:
            self.handle_error(request, client_address)
            self.shutdown_request(request)

    def process_request(self, request, client_address):
        req_id = "%032x" % (random.getrandbits(128))
        t = Thread(target=self.process_request_thread, args=(request, client_address), name=req_id)
        t.daemon = self.daemon_threads
        t.start()

    def handle_error(self, request, client_address):
        import traceback
        if "socket.py" not in traceback.format_exc():
            logger.error()


class Handler(BaseHTTPRequestHandler):
    def log_message(self, frm, *args):
        pass

    def do_POST(self):
        """
        Peticiones post se utilizan para obtener ajustes
        :return:
        """
        length = int(self.headers.getheader('content-length'))
        data = jsontools.load_json(self.rfile.read(length))
        value = jsontools.dump_json(settings.get_setting(data['id'], data['module']) == data['value'])
        self.send_response(200)
        self.send_header('Content-type', 'text/html; charset=utf-8')
        self.send_header('Content-length', str(len(value)))
        self.end_headers()
        self.wfile.write(value)
        self.wfile.close()

    def do_GET(self):
        cookies = self.headers.getheader('Cookie')
        if cookies:
            cookies = dict([(c.split('=')[0], c.split('=')[1]) for c in cookies.split(';')])
        else:
            cookies = {}
        skin = cookies.get('skin', 'default')

        if self.path in ('', '/'):

            # Control de accesos
            username = "user"
            password = "password"
            access_control = False

            # Comprueba la clave
            if access_control and self.headers.getheader('Authorization') != "Basic " + base64.b64encode(
                                    username + ":" + password):
                self.send_response(401)
                self.send_header('WWW-Authenticate',
                                 'Basic realm=\"Introduce el nombre de usuario y clave para acceder a MediaExplorer\"')
                self.send_header('Content-type', 'text/html; charset=utf-8')
                self.end_headers()
                self.wfile.write('¡Los datos introducidos no son correctos!')
                return

            f = open(os.path.join(sysinfo.runtime_path, 'resources', 'skin', skin, 'main.html'), 'rb')
            self.send_response(200)
            self.send_header('Content-type', 'text/html')
            self.end_headers()
            respuesta = f.read()
            respuesta = respuesta.replace(
                "{$WSPORT}",
                str(settings.get_setting("websocket_port", platformsettings.__file__))
            )
            self.wfile.write(respuesta)
            f.close()

        if self.path.startswith('/media/'):
            file_path = os.path.join(sysinfo.runtime_path, "resources", "skin", skin, self.path[7:])
            from mimetypes import MimeTypes
            mime = MimeTypes()
            mime_type = mime.guess_type(file_path)
            try:
                mim = mime_type[0]
            except Exception:
                mim = ""
            f = open(file_path, "rb")
            self.send_response(200)
            self.send_header('Content-type', mim)
            self.end_headers()
            self.wfile.write(f.read())
            f.close()

        if self.path.startswith('/local/') or self.path.startswith('/data/'):
            if self.path.startswith('/data/'):
                file_path = filetools.join(sysinfo.data_path, *urllib.unquote_plus(self.path[6:]).split('/'))
            else:
                file_path = filetools.encode(urllib.unquote_plus(self.path[7:]))

            file_size = int(os.path.getsize(file_path))

            f = open(file_path, "rb")

            range_start, range_end = scrapertools.find_single_match(
                self.headers.get('range', ''),
                '^bytes=(\d*)-(\d*)'
            ) or ['','']
            if not range_start:
                range_start = 0
            else:
                range_start = int(range_start)

            if not range_end:
                range_end = file_size - 1
            else:
                range_end = int(range_end)


            self.send_response(206)
            self.send_header("Content-Disposition", "attachment; filename=video.mp4")
            self.send_header('Accept-Ranges', 'bytes')
            self.send_header('Content-Length', str(range_end - range_start + 1))
            self.send_header('Content-Range', "bytes %s-%s/%s" %(range_start, range_end, file_size))
            self.send_header("Connection", "close")

            self.end_headers()
            f.seek(range_start)
            while True:
                time.sleep(0.2)
                buff = f.read(1024 * 250)
                if not buff:
                    break
                self.wfile.write(buff)
            self.wfile.close()
            f.close()

        if self.path.startswith('/proxy/'):
            url = self.path.replace("/proxy/", "").split("/")[0]
            url = base64.b64decode(urllib.unquote_plus(url))

            request_headers = self.headers.dict

            if "host" in request_headers: request_headers.pop("host")
            if "referer" in request_headers: request_headers.pop("referer")
            if "cookie" in request_headers: request_headers.pop("cookie")

            if "|" in url:
                url_headers = dict(
                    [[header.split("=")[0].lower(), urllib.unquote_plus("=".join(header.split("=")[1:]))] for header in
                     url.split("|")[1].split("&")])
                url = url.split("|")[0]
                request_headers.update(url_headers)

            req = urllib2.Request(url, headers=request_headers)
            opener = urllib2.build_opener(urllib2.HTTPHandler(debuglevel=0))

            try:
                h = opener.open(req)
            except urllib2.HTTPError, e:
                h = e
            except:
                self.send_response("503")
                self.wfile.close()

            self.send_response(h.getcode())
            for header in h.info():
                self.send_header(header, h.info()[header])

            self.end_headers()

            blocksize = 1024
            bloqueleido = h.read(blocksize)
            while len(bloqueleido) > 0:
                self.wfile.write(bloqueleido)
                bloqueleido = h.read(blocksize)

            self.wfile.close()
            h.close()

    def address_string(self):
        return self.client_address[:2][0]


server = MyHTTPServer(
    ('', int(settings.get_setting("http_port", platformsettings.__file__)))
    , Handler
)


def start():
    threading.Thread(target=server.serve_forever).start()


def stop():
    server.socket.close()
    server.shutdown()
