# -*- coding: utf-8 -*-
import random
import threading

import WebSocketServer

from core import launcher
from core.libs import *
from platformcode import mediaserver


class HandleWebSocket(WebSocketServer.WebSocket):
    def __init__(self, *args, **kwargs):
        WebSocketServer.WebSocket.__init__(self, *args, **kwargs)
        self.req_id = "%032x" % (random.getrandbits(128))

    def handleMessage(self):
        threading.Thread(target=self.handleMessage_, args=[str(self.data)], name=self.req_id).start()

    def handleMessage_(self, data):
        json_message = jsontools.load_json(data)
        try:
            if "request" in json_message:
                mediaserver.clients[self.req_id]['handle'] = json_message['handle']

                if json_message['request']:
                    item = Item().fromurl(json_message['request'])
                else:
                    item = Item(channel="channelselector", action="mainlist", content_type="icons")
                launcher.run(item)
                mediaserver.endOfScript()

            elif "data" in json_message:
                mediaserver.clients[self.req_id]['data'] = json_message["data"]

            elif "action" in json_message:
                if json_message["action"] == 'restart':
                    os._exit(10)
                if json_message["action"].get('type') == 'error':
                    logger.debug('Se ha producido un error en Javascript:')
                    logger.debug(json_message["action"])

        except Exception:
            logger.error()

    def handleConnected(self):
        threading.Thread(target=self.handleConnected_, name=self.req_id).start()

    def handleConnected_(self):
        mediaserver.clients[self.req_id] = {'socket': self, 'handle': 0, 'data': {}, 'play_id': None, 'adult_mode': 0}
        try:
            if launcher.start():
                mediaserver.handleConnected(sysinfo.main_version)
        except Exception:
            logger.error()

    def handleClose(self):
        if self.req_id in mediaserver.clients:
            del mediaserver.clients[self.req_id]


server = WebSocketServer.SimpleWebSocketServer(
    '',
    int(settings.get_setting('websocket_port', platformsettings.__file__)),
    HandleWebSocket
)


def start():
    Thread(target=server.serveforever).start()


def stop():
    server.close()
