# -*- coding: utf-8 -*-
import mediaserver

from core.libs import *


class FirstRun(object):
    def __init__(self):
        logger.trace()
        self.restart = False
        self.step = 0
        self.id = None

    def set_step(self):
        data = {
            'action': "FirstRun.update",
            'dialog_id': self.id,
            'button1': '',
            'button2': '',
            'button3': ''
        }
        if self.step == 0:
            data['label'] = '<b>¡Bienvenido a MediaExplorer!</b>'
            data['text'] = ('Con este addon podrá disfrutar de sus películas y series favoritas navegando de manera fác'
                            'il, sencilla y segura desde su navegador, Smart TV, etc...</br></br>'
                            'Busque entre las diferentes webs disponibles de manera simultánea.</br>'
                            'Añada accesos directos a las diferentes secciones o contenidos de MediaExplorer en los \'F'
                            'avoritos\' para tenerlos siempre a mano.</br>'
                            'Visualice en pocos segundos las novedades añadidas en sus canales preferidos.</br>'
                            'Comparta un listado de enlaces recomendados con sus familiares y amigos.</br>'
                            'Y muchas mas funcionalidades que le invitamos a descubrir.</br></br>'
                            'Dediquemos un momento a configurar MediaExplorer antes de comenzar.</br></br>'
                            'Haga click en Empezar cuando esté preparado.')
            data['button1'] = 'Empezar'
            data['focus'] = 'button1'

        elif self.step == 1:
            data['label'] = '<b>Aviso legal</b>'
            msg = ('1.- El material disponible a través de MediaExplorer, ha sido recolectado de sitios webs que son pú'
                   'blicos (hdfull.tv, plusdede.com, seriesblanco.com, etc...), organizados como canales. Por lo que es'
                   'te material es considerado de libre distribución y su uso indebido no será nunca responsabilidad de'
                   ' los creadores o colaboradores de MediaExplorer.</br>'
                   '2.- Todos los vídeos que figuran en este addon son solo enlaces que han sido obtenidos de páginas c'
                   'omo powvideo.net, openload.co, youtube.com, netu.tv, etc. Por lo tanto MediaExplorer solo se limita'
                   ' a facilitar la visibilidad de los vídeos a través de los enlaces de estos y otros servidores.</br>'
                   '3.- MediaExplorer, no aloja vídeos ni distribuye ningún tipo de archivo para streaming o descarga. '
                   'El único material que proporciona son enlaces de webs de libre distribución. Por consiguiente solo '
                   'los canales y servidores son plenamente responsables de los contenidos que publiquen.</br>'
                   '4.- Los canales y servidores podrían, en algún caso, publicar vídeos con contenido no apto. MediaEx'
                   'plorer cuenta con un apartado exclusivo para adultos (por defecto deshabilitado), pero no puede gar'
                   'antizar que todo el material de este tipo se filtre correctamente. Usted es responsable de proteger'
                   ' a los menores de esta exposición.</br>'
                   '5.- Si este tipo de contenido esta prohibido en su país, solamente usted es responsable del uso de '
                   'MediaExplorer.</br></br>'
                   'Si ha leído, comprendido y acepta estas condiciones pulse Aceptar.</br>'
                   'En caso contrario, pulse Cancelar')
            data['text'] = msg
            data['button1'] = 'Aceptar'
            data['button2'] = 'Cancelar'
            data['focus'] = 'button1'

        elif self.step == 2:
            data['label'] = '<b>Normas de uso y distribución</b>'
            data['text'] = 'MediaExplorer es un software <b>gratuito</b> con licencia GPLv3.</br></br>' \
                           'No obstante, no se permite su distribuición preinstalada en ningún tipo ' \
                           'de dispositivo ni tampoco su inclusión total o parcial en cualquier ' \
                           'paquete de software y/o hardware por el que el usuario final deba pagar ' \
                           'su adquisición, uso o asistencia.</br></br>' \
                           'Tampoco se permite ninguna modificación del codigo que tenga por objeto ' \
                           'contravenir el significado del párrafo anterior o incluir imágenes, ' \
                           'vídeos o sonidos sujetos a derechos de autor y que pudieran incumplir ' \
                           'el <b>Aviso Legal antes aceptado.</b></br></br>' \
                           'Si cree que alguna de estas normas no se está cumpliendo póngase en ' \
                           'contacto con nosotros.'

            data['button1'] = 'Continuar'
            data['focus'] = 'button1'

        elif self.step == 3:
            opciones_disponibles = []
            opciones_no_disponibles = []

            try:
                import libtorrent
                opciones_disponibles.append('&nbsp;&nbsp;&nbsp;&nbsp;Reproductor Torrent')
            except Exception:
                opciones_no_disponibles.append('&nbsp;&nbsp;&nbsp;&nbsp;Reproductor Torrent')

            try:
                from megaserver import client

                opciones_disponibles.append('&nbsp;&nbsp;&nbsp;&nbsp;Soporte para servidor Mega nativo')
            except Exception:
                opciones_no_disponibles.append('&nbsp;&nbsp;&nbsp;&nbsp;Soporte para servidor Mega nativo')

            data['label'] = '<b>Librerías externas</b>'
            msg = ('MediaExplorer se distribuye preparado para funcionar. Pero puede interactuar con otras librerías o '
                   'añadidos externos para una mejor experiencia.</br></br>')

            if opciones_disponibles:
                msg += 'Funciones disponibles:</br>%s</br></br>' % '</br>'.join(opciones_disponibles)

            if opciones_no_disponibles:
                msg += 'Funciones no disponibles:</br>%s</br>' % '</br>'.join(opciones_no_disponibles)

            data['text'] = msg
            data['button1'] = 'Continuar'
            data['focus'] = 'button1'

        elif self.step == 4:
            data['label'] = '<b>¡Ya hemos terminado!</b>'
            data['text'] = ('MediaExplorer ya está listo para ser utilizado.</br></br>'
                            'Aunque le recomendamos que compruebe si hay novedades disponibles en el apartado \'Actuali'
                            'zaciones\'.</br></br>' 
                            'También puede mejorar el funcionamiento de MediaExplorer entrando en las diferentes seccio'
                            'nes de \'Ajustes\'.</br></br>'
                            '¡Esperamos que lo disfrute!')

            data['button1'] = 'Finalizar'
            data['focus'] = 'button1'

        else:
            settings.set_setting('first_run', True)
            data = None

        return data

    def start(self):
        logger.trace()
        self.doModal()

        if self.step == 1:
            return False

        if self.restart:
            platformtools.dialog_ok(
                'MediaExplorer',
                'Debe reiniciar MediaExplorer para que funcione correctamente.')

        return True

    def doModal(self):
        self.id = mediaserver.send_message({
            "action": 'FirstRun'
        })
        data = self.set_step()
        while data:
            mediaserver.send_message(data)
            resp = mediaserver.whait_response(self.id)
            if resp['action'] == 'button1':
                self.step += 1
                data = self.set_step()

            if resp['action'] == 'button2':
                if self.step == 1:
                    break
                else:
                    self.step += 1
                    data = self.set_step()

        self.close()

    def close(self):
        mediaserver.send_message({
            "action": 'FirstRun.close',
            "dialog_id": self.id
        })
