import threading
import time

clients = {}


class ListItem:
    def __init__(self, label=None):
        self.label = label
        self.info = {}
        self.art = {}
        self.context = []

    def setInfo(self, _type, info):
        for k, v in info.items():
            if type(v) == str:
                info[k] = v.replace('\n', '<br/>')
        self.info.update(info)

    def setArt(self, art):
        self.art.update(art)

    def addContextMenuItems(self, context):
        self.context = context

    def to_dict(self):
        return {
            'label': self.label,
            'info': self.info,
            'art': self.art,
            'context': self.context
        }


class DialogProgress:
    def __init__(self):
        self.id = None
        self.closed = False
        self.canceled = False
        self.canceled = False

    def create(self, heading, line1, line2='', line3=''):
        json_data = {
            'action': 'DialogProgress.create',
            'heading': heading,
            'line1': line1,
            'line2': line2,
            'line3': line3
        }
        self.id = send_message(json_data)

    def update(self, percent, line1='', line2='', line3=''):
        json_data = {
            'action': 'DialogProgress.update',
            'percent': percent,
            'line1': line1,
            'line2': line2,
            'line3': line3,
            'dialog_id': self.id
        }
        send_message(json_data)

    def close(self):
        self.canceled = self.iscanceled()
        json_data = {
            'action': 'DialogProgress.close',
            'dialog_id': self.id
        }
        send_message(json_data)
        self.closed = True

    def iscanceled(self):
        if self.closed:
            return self.canceled

        json_data = {
            'action': 'DialogProgress.iscanceled',
            'dialog_id': self.id
        }
        req_id = send_message(json_data)
        return whait_response(req_id)


class DialogProgressBG:
    def __init__(self):
        self.id = None
        self.closed = False

    def create(self, heading, message):
        self.closed = False
        json_data = {
            'action': 'DialogProgressBG.create',
            'heading': heading,
            'message': message,
        }
        self.id = send_message(json_data)

    def isFinished(self):
        return self.closed

    def update(self, percent=0, heading="", message=""):
        json_data = {
            'action': 'DialogProgressBG.update',
            'percent': percent,
            'heading': heading,
            'message': message,
            'dialog_id': self.id
        }
        send_message(json_data)

    def close(self):
        json_data = {
            'action': 'DialogProgressBG.close',
            'dialog_id': self.id
        }
        send_message(json_data)
        self.closed = True


class Dialog:
    def __init__(self):
        pass

    def ok(self, heading, line1, line2=None, line3=None):
        req_id = send_message({
            'action': 'Dialog.ok',
            'heading': heading,
            'line1': line1,
            'line2': line2,
            'line3': line3
        })

        whait_response(req_id)

    def notification(self, heading, message, icon, t, sound):
        req_id = send_message({
            'action': 'Dialog.notification',
            'heading': heading,
            'message': message,
            'icon': icon,
            'time': t,
            'sound': sound
        })

    def yesno(self, heading, line1, line2=None, line3=None, nolabel='No', yeslabel='Si', autoclose=None):
        req_id = send_message({
            'action': 'Dialog.yesno',
            'heading': heading,
            'line1': line1,
            'line2': line2,
            'line3': line3,
            'nolabel': nolabel,
            'yeslabel': yeslabel,
            'autoclose': autoclose
        })

        return whait_response(req_id)

    def img_yesno(self, img, heading, message, noshow, yeslabel, nolabel):
        print heading
        req_id = send_message({
            'action': 'Dialog.img_yesno',
            'img': img,
            'heading': heading,
            'message': message,
            'noshow': noshow,
            'yeslabel': yeslabel,
            'nolabel': nolabel
        })

        return whait_response(req_id)

    def select(self, heading, _list):
        req_id = send_message({
            'action': 'Dialog.select',
            'heading': heading,
            'list': _list
        })

        return whait_response(req_id)

    def numeric(self, _type, heading, default=''):
        req_id = send_message({
            'action': 'Dialog.numeric',
            'heading': heading,
            'type': _type,
            'default': default
        })

        return whait_response(req_id)


class Keyboard:
    def __init__(self, default, heading, hidden):
        self.default = default
        self.heading = heading
        self.hidden = hidden
        self.response = None

    def doModal(self):
        req_id = send_message({
            'action': 'Keyboard',
            'heading': self.heading,
            'default': self.default,
            'hidden': self.hidden
        })

        self.response = whait_response(req_id)

    def isConfirmed(self):
        return self.response is not None

    def getText(self):
        return self.response or ''

    def setDefault(self, default):
        self.default = default

    def setHeading(self, heading):
        self.heading = heading

    def setHiddenInput(self, hidden):
        self.hidden = hidden


def handleConnected(version):
    send_message({
        'action': 'handleConnected',
        'version': version
    })


def addDirectoryItem(handle, url, listitem, isFolder=False, totalItems=None):
    send_message({
        'action': 'addDirectoryItem',
        'handle': handle,
        'url': url,
        'listitem': listitem.to_dict(),
        'isFolder': isFolder,
        'totalItems': totalItems
    })


def setPluginCategory(handle, category):
    send_message({
        'action': 'setPluginCategory',
        'handle': handle,
        'category': category
    })


def endOfScript():
    send_message({
        'action': 'endOfScript'
    })


def containerRefresh():
    send_message({
        'action': 'containerRefresh'
    })


def endOfDirectory(handle, succeeded=True):
    send_message({
        'handle': handle,
        'action': 'endOfDirectory',
        'succeeded': succeeded
    })


def setViewMode(view):
    send_message({
        'action': 'setViewMode',
        'view': view
    })


def play(url, listitem):
    req_id = send_message({
        'action': 'play',
        'url': url,
        'listitem': listitem.to_dict()
    })
    clients[threading.current_thread().name]['play_id'] = req_id


def isPlaying():
    if not clients[threading.current_thread().name]['play_id']:
        return False
    req_id = send_message({
        'action': 'isPlaying',
        'play_id': clients[threading.current_thread().name]['play_id']
    })
    return whait_response(req_id)


def get_handle():
    if not threading.current_thread().name in clients:
        return None
    return clients[threading.current_thread().name]['handle']


def send_message(data):
    if not threading.current_thread().name in clients:
        return None
    client = clients[threading.current_thread().name]['socket']
    import random
    from core import jsontools
    req_id = "%032x" % (random.getrandbits(128))
    data["id"] = req_id
    client.sendMessage(jsontools.dump_json(data))
    return req_id


def whait_response(req_id):
    if not threading.current_thread().name in clients:
        return None

    while not clients[threading.current_thread().name]['data'].get('id') == req_id:
        time.sleep(0.1)
    value = clients[threading.current_thread().name]['data']["result"]
    clients[threading.current_thread().name]['data'] = {}
    return value


def set_adult_client(value=True):
    clients[threading.current_thread().name]['adult_mode'] = int(value)

def get_adult_client():
    return clients[threading.current_thread().name]['adult_mode']