# -*- coding: utf-8 -*-
from core.libs import *
from platformcode import mediaserver
from platformcode import viewtools


def dialog_ok(heading, line1, line2="", line3=""):
    """
    Muestra un diálogo
    :param heading:
    :param line1:
    :param line2:
    :param line3:
    :return:
    """
    dialog = mediaserver.Dialog()
    return dialog.ok(heading, line1, line2, line3)


def dialog_notification(heading, message, icon=0, t=5000, sound=True):
    """
    Muestra una notificaión
    :param heading:
    :param message:
    :param icon:
    :param t:
    :param sound:
    :return:
    """
    dialog = mediaserver.Dialog()
    dialog.notification(heading, message, icon, t, sound)


def dialog_yesno(heading, line1, line2="", line3="", nolabel="No", yeslabel="Si", autoclose=None):
    """
    Muestra un dialogo con dos opciones (si y no)
    :param heading:
    :param line1:
    :param line2:
    :param line3:
    :param nolabel:
    :param yeslabel:
    :param autoclose:
    :return:
    """
    dialog = mediaserver.Dialog()
    return dialog.yesno(heading, line1, line2, line3, nolabel, yeslabel, autoclose)


def dialog_img_yesno(img, heading="", message="", no_show = True, yes_label ='Aceptar', no_label='Cancelar'):
    """
    Muestra un dialogo con una imagen y dos opciones (si y no)
    :param heading: Titulo del cuadro de dialogo
    :param message: Texto multilinea
    :param no_show: Mostrar o no el boton NO
    :param yeslabel: Etiqueta boton YES
    :param nolabel: Etiqueta boton NO
    :return: True si se pulsa el boton YES, False en caso contrario.
    """
    dialog = mediaserver.Dialog()
    return dialog.img_yesno(img, heading, message, no_show, yes_label, no_label)


def dialog_select(heading, _list):
    """
    Muestra un cuadro de selección
    :param heading:
    :param _list:
    :return:
    """
    return mediaserver.Dialog().select(heading, _list)


def dialog_progress(heading, line1, line2="\n", line3="\n"):
    """
    Muestra un dialogo de progreso
    :param heading:
    :param line1:
    :param line2:
    :param line3:
    :return:
    """
    dialog = mediaserver.DialogProgress()
    dialog.create(heading, line1, line2, line3)
    return dialog


def dialog_progress_bg(heading, message=""):
    """
    Muestra un dialogo de progresoen segundo plano
    :param heading:
    :param message:
    :return:
    """
    dialog = mediaserver.DialogProgressBG()
    dialog.create(heading, message)
    return dialog


def dialog_input(default="", heading="", hidden=False):
    """
    Muestra un diálogo para introducir texto
    :param default:
    :param heading:
    :param hidden:
    :return:
    """
    keyboard = mediaserver.Keyboard(default, heading, hidden)
    keyboard.doModal()
    if keyboard.isConfirmed():
        return keyboard.getText()
    else:
        return None


def dialog_numeric(_type, heading, default=""):
    """
    Muestra un diálogo para introducir numeros
    :param _type:
    :param heading:
    :param default:
    :return:
    """
    dialog = mediaserver.Dialog()
    d = dialog.numeric(_type, heading, default)
    return d


def show_settings(controls=None, title="", callback=None, item=None, custom_button=None, mod=None):
    """
    Muestra la ventana de configuración
    :param controls:
    :param title:
    :param callback:
    :param item:
    :param custom_button:
    :param mod:
    :return:
    """
    from settings_window import SettingsWindow
    return SettingsWindow().start(
        controls=controls,
        title=title,
        callback=callback,
        item=item,
        custom_button=custom_button,
        mod=mod
    )


def show_info(item, title=""):
    """
    Muestra una ventana con la información del item (movie, tvshow, season, episode) en caso de que
    el item sea un list con varios, permite elegir uno y devuelve el index del item seleccionado
    :param item:
    :param title:
    :return:
    """
    from info_window import InfoWindow
    return InfoWindow().start(item, title=title)


def show_recaptcha(key, referer):
    """
    reCaptcha
    :param key:
    :param referer:
    :return:
    """
    # TODO: No funciona ya que ahora hay que usar la v2 de la api de google
    from recaptcha import Recaptcha
    return Recaptcha().start(key, referer)


def show_captcha(url):
    """
    Muetra una imagen con el captcha y devuelve el texto escrito por el usuario
    :param url:
    :return:
    """
    from captcha import Captcha
    return Captcha().start(url)


def show_first_run():
    from firstrun import FirstRun
    return FirstRun().start()


def media_info(item):
    show_info(item, 'informacion')


def itemlist_refresh():
    """
    Actualiza la ventana recargando el mismo item
    :return:
    """
    return mediaserver.containerRefresh()


def itemlist_update(item):
    """
    Actualiza la ventana cargando el item pasado
    :param item:
    :return:
    """
    mediaserver.containerRefresh(item.tourl())


def get_profile():
    """
    Obtiene el perfil y los datos
    :return:
    """
    logger.trace()

    profiles_path = os.path.join(sysinfo.runtime_path, 'resources', 'profiles')
    saved_profile = settings.get_setting('color_profile', viewtools.__file__)

    # Si tenemos un perfil configurado, comprobamos si vale para nuestro skin
    if saved_profile:
        try:
            profile_data = jsontools.load_file(os.path.join(profiles_path, saved_profile))
        except Exception:
            logger.error()
        else:
            return saved_profile, profile_data

    # Si no tenemos perfil para nuestro skin, cargamos el básico
    return 'default.json', jsontools.load_file(os.path.join(profiles_path, 'default.json'))


def get_profile_list():
    """
    Obtiene la lista de perfiles para el skin seleccionado
    :return:
    """
    logger.trace()
    itemlist = list()

    profiles_path = os.path.join(sysinfo.runtime_path, 'resources', 'profiles')

    for fname in os.listdir(profiles_path):
        if not fname.endswith('.json'):
            continue
        try:
            profile_data = jsontools.load_file(os.path.join(profiles_path, fname))
        except Exception:
            logger.error()
        else:
            itemlist.append([fname, profile_data['name']])

    return itemlist


def render_items(itemlist, parent_item):
    """
    Muestra los items del itemlist en la pantalla
    :param itemlist:
    :param parent_item:
    :return:
    """
    logger.trace()

    # Si no es un list no hay nada que mostrar, salimos
    if not type(itemlist) == list:
        return

    # Si no hay items mostramos un aviso
    if not len(itemlist):
        itemlist.append(
            Item(
                label="No hay elementos que mostrar"
            )
        )
        parent_item.content_type = 'items'

    # Obtener información en el listado
    if settings.get_setting('itemlist_info', viewtools.__file__) and not parent_item.channel == 'library' and \
            not (parent_item.channel == 'newest' and settings.get_setting("group_result", 'core/newest') == 3) and \
            not (parent_item.channel == 'finder' and settings.get_setting("group_result", 'core/finder') == 3):

        dialog = platformtools.dialog_progress_bg('Obteniendo información de medios')
        for ready, total in mediainfo.get_itemlist_info(
                itemlist,
                extended=settings.get_setting('extended_info', viewtools.__file__),
                ask=False
        ):
            dialog.update(ready * 100 / total, 'Obteniendo información de medios', '%s de %s' % (ready, total))

        dialog.close()

    if settings.get_setting('media_info', viewtools.__file__) and parent_item.type in (
            'movie', 'tvshow', 'season', 'episode'):
        i = parent_item.clone(label='Información', action='info', channel=None, thumb='', fanart='')
        itemlist.insert(0, i)
        if i.type == 'movie':
            mediainfo.get_movie_info(i, extended=True, ask=False)
        if i.type == 'tvshow':
            mediainfo.get_tvshow_info(i, extended=True, ask=False)
        if i.type == 'season':
            mediainfo.get_season_info(i, extended=True, ask=False)
        if i.type == 'episode':
            mediainfo.get_episode_info(i, extended=True, ask=False)
        i.type = 'info'

    # Recorremos el itemlist
    for item in itemlist:
        # Si el item está marcado como adulto pero no está activado en la configuración lo saltamos
        if item.adult and not settings.get_setting('adult_mode'):
            continue

        # Si el item es del tipo label eliminamos la accion
        if item.type == 'label':
            if item.action: del item.action
            if item.context_action: del item.context_action

        if parent_item.action == 'mainlist' and os.path.isfile(os.path.join(
                sysinfo.runtime_path,
                'resources',
                'images',
                'fanart',
                '%s.jpg' % item.channel
        )):
            item.fanart = 'fanart/%s.jpg' % item.channel

        # Creamos el listitem
        listitem = mediaserver.ListItem(viewtools.set_label_format(item))

        # Añadimos la información
        viewtools.set_item_info(item, listitem)
        viewtools.set_art(item, parent_item, listitem)

        # Añadimos el menú contextual
        listitem.addContextMenuItems(set_context_commands(item, parent_item))

        # Añadimos el item al listitem
        mediaserver.addDirectoryItem(
            handle=mediaserver.get_handle(),
            url=item.tourl(),
            listitem=listitem,
            isFolder=item.folder,
            totalItems=len(itemlist)
        )

    # Seleccionamos el tipo de vista
    viewtools.set_viewmode(parent_item)

    # Añadimosel nombre del canal
    mediaserver.setPluginCategory(
        handle=mediaserver.get_handle(),
        category=parent_item.name
    )

    # Cerramos
    mediaserver.endOfDirectory(handle=mediaserver.get_handle(), succeeded=True)


def set_context_commands(item, parent_item):
    """
    Crea los elementos del menu contextual
    :param item:
    :param parent_item:
    :return:
    """
    logger.trace()
    context_commands = []

    # Opciones segun item.context
    for command in item.context:
        if type(command) == dict:
            if "goto" in command:
                context_commands.append(
                    (
                        command["label"],
                        item.clone(**command).tourl()

                    )
                )
            else:
                context_commands.append(
                    (
                        command["label"],
                        item.clone(**command).tourl()

                    )
                )

    # Abrir configuración
    context_commands.append(
        (
            "Abrir configuración",
            Item(
                channel='settings',
                action="mainlist",
                thumb='thumb/setting.png',
                icon='icon/setting.png',
                poster='poster/setting.png'
            ).tourl()
        )
    )

    # Biblioteca
    if not item.channel == 'library':
        # Añadir película a la biblioteca
        if item.type == 'movie':
            context_commands.append(
                (
                    "Añadir Película a la biblioteca",
                    item.clone(
                        context_channel='library',
                        context_action='add_to_library'
                    ).tourl()
                )
            )

        # Añadir serie a la bilioteca
        if item.type == 'tvshow':
            context_commands.append(
                (
                    "Añadir Serie a la biblioteca",
                    item.clone(
                        context_channel='library',
                        context_action='add_to_library'
                    ).tourl()
                )
            )

        # Añadir vídeo a la biblioteca
        if item.type == 'video':
            context_commands.append(
                (
                    "Añadir Vídeo a la biblioteca",
                    item.clone(
                        context_channel='library',
                        context_action='add_to_library'
                    ).tourl()
                )
            )

    # Descargar película
    if item.type == 'movie':
        context_commands.append(
            (
                "Descargar película",
                item.clone(
                    context_channel='downloads',
                    context_action='add_to_downloads'
                ).tourl()
            )
        )

    # Descargar serie
    if item.type == 'tvshow':
        context_commands.append(
            (
                "Descargar serie",
                item.clone(
                    context_channel='downloads',
                    context_action='add_to_downloads'
                ).tourl()
            )
        )

    # Descargar vídeo
    if item.type == 'video':
        context_commands.append(
            (
                "Descargar vídeo",
                item.clone(
                    context_channel='downloads',
                    context_action='add_to_downloads'
                ).tourl()
            )
        )

    # Ir al Menu Principal (channel.mainlist)
    if parent_item.action != 'mainlist':
        context_commands.append(
            (
                "Ir al Menu Principal",
                Item(
                    channel=parent_item.channel,
                    action="mainlist"
                ).tourl()
            )
        )

    # Información del canal
    if item.type == 'channel':
        context_commands.append(
            (
                "Información del canal",
                item.clone(context_channel='channelselector',
                           context_action='channel_info').tourl()
            )
        )

    # Autoplay
    if item.type in ['movie', 'episode']:
        if not settings.get_setting('autoplay'):
            context_commands.append(
                (
                    "Autoplay",
                    item.clone(context_autoplay=True).tourl()
                )
            )
        else:
            context_commands.append(
                (
                    "Ver servidores",
                    item.clone(context_autoplay=False).tourl()
                )
            )

    # Añadir a Mi Lista
    if item.type in ['movie', 'tvshow'] and not item.list_id:
        context_commands.append(
            (
                "Añadir a Mi Lista",
                item.clone(context_channel='lists', context_action='add_to_my_list').tourl()
            )
        )

    # Buscar
    if item.type in ('season', 'episode', 'tvshow', 'movie'):
        context_commands.append(
            (
                "Buscar '%s'" % (item.tvshowtitle or item.title),
                item.clone(
                    context_channel='finder',
                    context_action='search',
                    search_categories={'movie': True} if item.type == 'movie' else {'tvshow': True},
                    content_type='movies' if item.type == 'movie' else 'tvshows',
                    query=item.tvshowtitle or item.title,
                ).tourl()
            )
        )

    # Información del medio
    if item.type in ('season', 'episode', 'tvshow', 'movie', 'server'):
        context_commands.append(
            (
                "Información",
                item.clone(
                    context_channel='launcher',
                    context_action='info'
                ).tourl()
            )
        )

    # Favoritos
    if item.type in ('season', 'episode', 'tvshow', 'movie', 'channel'):
        if parent_item.channel != 'bookmarks':
            context_commands.append(
                (
                    "Añadir a favoritos",
                    item.clone(
                        context_channel='bookmarks',
                        context_action='add_bookmark'
                    ).tourl()

                )
            )

    return sorted(context_commands, key=lambda comand: comand[0])


def is_playing():
    """
    Compruena si se está reproducioendo
    :return:
    """
    return mediaserver.isPlaying()


def stop_video():
    """
    Deteniene la reproducción del video
    :return:
    """
    mediaserver.stop()


def play(item):
    """
    Muestra el dialogo con las opciones de reproducción y lanza la función adecuada segun la opcion selecionada
    :param item:
    :return:
    """
    logger.trace()

    # Ponemos el tipo correcto
    item = item.clone(type=item.mediatype)

    # Buscar información
    if settings.get_setting('play_info', viewtools.__file__):
        from core import mediainfo
        if item.type == 'movie':
            mediainfo.get_movie_info(item, True)
        if item.type == 'episode':
            mediainfo.get_episode_info(item, True)

    # Si el item no tiene video_urls se asignan
    if not item.video_urls:
        if item.server:
            item.video_urls = servertools.resolve_video_urls(item)
        else:
            item.video_urls = [Video(url=item.url, server='Directo')]

    # Si video_urls es un str es que hay un error, se muestra
    if type(item.video_urls) == str:
        if not item.autoplay:
            dialog_ok('No puedes ver el video', item.video_urls)
        return

    # Cargamos las opciones devueltas por el servidor ordenadas por resolucion
    options = [
        item.clone(
            action=('play_video', 'play_torrent')[item.server == 'torrent'],
            url=v.url,
            label='Ver el video: [%s]%s [%s]' % (v.type, ('', ' [%s]' % v.res)[bool(v.res)], v.server),
            channel='',
            subtitle=v.subtitle or item.subtitle,
            mpd=v.mpd,
            headers=v.headers
        )
        for v in item.video_urls  # Las video_url deben estar ordenadas de mejor a peor calidad
    ]

    # Añadimos otras funciones en funcion de ciertos parametros
    if item.channel == 'downloads':
        options.append(item.clone(
            action='remove',
            channel='downloads',
            label='Eliminar archivo',
            title=item.title or item.label,
        ))

    # Si hay mas de una opcion y no venimos de autoplay...
    if len(options) > 1 and not item.autoplay:
        # ... abrimos el menu de selección
        index = dialog_select('Selecciona una opcion', [o.label for o in options])
        if index == -1:
            return
    else:
        # ...sino seleccionamos la unica opcion posible
        index = 0

    # Si el action está en otro canal, lo lanzamos mediante el launcher
    if options[index].channel:
        from core import launcher
        launcher.run(options[index].clone())

    # En caso contrario, lanzamos la funcion aquí
    else:
        getattr(sys.modules[__name__], options[index].action)(options[index])


def play_video(item):
    """
    Lanza el reproductor de kodi con el vídeo seleccionado
    :param item:
    :return:
    """
    logger.trace()

    # Creamos el listItem
    listitem = mediaserver.ListItem()

    # Asignamos los datos
    viewtools.set_item_info(item, listitem)
    listitem.setArt({'fanart': item.fanart})
    listitem.setArt({'poster': item.poster})
    listitem.setArt({'banner': item.banner})
    listitem.setArt({'thumb': item.thumb})
    mediaserver.play(item.url, listitem)


def play_torrent(item):
    """
    Reproduce un torrent
    :param item:
    :return:
    """
    logger.trace()

    # Creamos el listItem
    listitem = mediaserver.ListItem()

    # Asignamos los datos
    viewtools.set_item_info(item, listitem)
    listitem.setArt({'fanart': item.fanart})
    listitem.setArt({'poster': item.poster})
    listitem.setArt({'banner': item.banner})
    listitem.setArt({'thumb': item.thumb})

    played = False
    debug = settings.get_setting("debug") == 3

    # Importamos el cliente
    from btserver import Client

    # Directorio torrent
    client_tmp_path = os.path.join(sysinfo.data_path, 'torrent')

    # Si no existe el directorio lo creamos
    if not os.path.isdir(client_tmp_path):
        os.makedirs(client_tmp_path)

    # Iniciamos el cliente:
    c = Client(
        url=item.url,
        is_playing_fnc=mediaserver.isPlaying,
        wait_time=None,
        timeout=10,
        temp_path=client_tmp_path,
        print_status=debug
    )

    # Mostramos el progreso
    dialog = dialog_progress("Torrent", "Iniciando...")

    # Mientras el progreso no sea cancelado ni el cliente cerrado
    while not c.closed:
        try:
            # Obtenemos el estado del torrent
            s = c.status
            if debug:
                # Montamos las tres lineas con la info del torrent
                txt = '%.2f%% de %.1fMB %s | %.1f kB/s' % (
                    s.progress_file,
                    s.file_size,
                    s.str_state,
                    s.download_rate
                )
                txt2 = 'S: %d(%d) P: %d(%d) | DHT:%s (%d) | Trakers: %d' % (
                    s.num_seeds,
                    s.num_complete,
                    s.num_peers,
                    s.num_incomplete,
                    s.dht_state,
                    s.dht_nodes,
                    s.trackers
                )
                txt3 = 'Origen Peers TRK: %d DHT: %d PEX: %d LSD %d ' % (
                    s.trk_peers,
                    s.dht_peers,
                    s.pex_peers,
                    s.lsd_peers
                )
            else:
                txt = '%.2f%% de %.1fMB %s | %.1f kB/s' % (
                    s.progress_file,
                    s.file_size,
                    s.str_state,
                    s.download_rate
                )
                txt2 = 'S: %d(%d) P: %d(%d)' % (
                    s.num_seeds,
                    s.num_complete,
                    s.num_peers,
                    s.num_incomplete
                )

                txt3 = 'Deteniendo automaticamente en: %s' % s.timeout

            dialog.update(s.buffer, txt, txt2, txt3)
            time.sleep(0.5)

            if dialog.iscanceled():
                dialog.close()
                if s.buffer == 100:
                    if dialog_yesno("Torrent", "¿Deseas iniciar la reproduccion?"):
                        played = False
                        dialog = dialog_progress("Torrent", "")
                        dialog.update(s.buffer, txt, txt2, txt3)
                    else:
                        dialog = dialog_progress("Torrent", "")
                        break

                else:
                    if dialog_yesno("Torrent", "¿Deseas cancelar el proceso?"):
                        dialog = dialog_progress("Torrent", "")
                        break

                    else:
                        dialog = dialog_progress("Torrent", "")
                        dialog.update(s.buffer, txt, txt2, txt3)

            # Si el buffer se ha llenado y la reproduccion no ha sido iniciada, se inicia
            if s.buffer == 100 and not played:
                # Cerramos el progreso
                dialog.close()

                # Iniciamos el reproductor
                mediaserver.play(c.get_play_list(), listitem)

                # Marcamos como reproducido para que no se vuelva a iniciar
                played = True

                # Y esperamos a que el reproductor se cierre
                while mediaserver.isPlaying():
                    time.sleep(1)

                # Cuando este cerrado,  Volvemos a mostrar el dialogo
                dialog = dialog_progress("Torrent", "")
                dialog.update(s.buffer, txt, txt2, txt3)

        except Exception:
            logger.error()
            break

    dialog.update(100, "Terminando y eliminando datos", " ", " ")

    # Detenemos el cliente
    if not c.closed:
        c.stop()

    # Y cerramos el progreso
    dialog.close()
