# -*- coding: utf-8 -*-
import base64
import inspect

import mediaserver
from core.libs import *


class SettingsWindow:
    def __init__(self):
        self.return_value = None
        self.req_id = None
        self.callback = None
        self.item = None
        self.custom_button = None
        self.module = None
        self.controls = None
        self.title = None

    def start(self, controls=None, title="Opciones", callback=None, item=None, custom_button=None, mod=None):
        logger.trace()

        self.controls = controls
        self.title = title
        self.callback = callback
        self.item = item
        self.custom_button = custom_button

        if not mod:
            self.module = inspect.currentframe().f_back.f_back.f_code.co_filename
        else:
            self.module = mod

        if not self.controls:
            self.controls = moduletools.get_controls(self.module)
        else:
            self.controls = list(self.controls)

        if not self.controls:
            return None

        if self.title == '':
            self.title = 'Ajustes'

        self.doModal()

        return self.return_value

    def doModal(self):
        json_data = {
            'action': 'SettingsWindow',
            'heading': self.title,
            'custom_button': self.custom_button,
            'callback': bool(self.callback),
            'module': self.module,
            'controls': []
        }

        ids = []
        for c in self.controls:
            for k, v in c.items():
                if type(v) == str and re.match('^eval\((.*?)\)$', v):
                    c[k] = eval(re.match('^eval\((.*?)\)$', v).group(1))
                
                if type(v) == list:
                    for x, d in enumerate(v):
                        if type(d) == str and re.match('^eval\((.*?)\)$', d):
                            v[x] = eval(re.match('^eval\((.*?)\)$', d).group(1))

            if 'type' not in c:
                c['type'] = 'label'

            if c.get('id') and c.get('id') in ids:
                continue

            if c.get('color'):
                c['color'] = self.color_translate(c['color'])

            if c["type"] == "bool":
                if not c.get('label'):
                    continue
                if not c.get('id'):
                    continue
                if c.get('value') is None:
                    c['value'] = settings.get_setting(c['id'], self.module)
                if not c.get('default'):
                    c['default'] = False

            elif c["type"] == 'text':
                if not c.get('label'):
                    continue
                if c.get('value') is None and c.get('id'):
                    c['value'] = settings.get_setting(c['id'], self.module)
                elif c.get('value') is None:
                    c['value'] = ''
                if not c.get('default'):
                    c['default'] = ''

            elif c["type"] == 'file':
                if not c.get('label'):
                    continue
                if not c.get('id'):
                    continue
                if c.get('value') is None:
                    c['value'] = settings.get_setting(c['id'], self.module)
                if not c.get('default'):
                    c['default'] = ''

            elif c["type"] == 'folder':
                if not c.get('label'):
                    continue
                if not c.get('id'):
                    continue
                if c.get('value') is None:
                    c['value'] = settings.get_setting(c['id'], self.module)
                if not c.get('default'):
                    c['default'] = ''

            elif c["type"] == 'list':
                if not c.get('label'):
                    continue
                if not c.get('id'):
                    continue
                if not type(c.get('lvalues')) == list:
                    continue
                if c.get('value') is None:
                    c['value'] = settings.get_setting(c['id'], self.module)
                if not c.get('default'):
                    c['default'] = 0

            elif c["type"] == 'label':
                if not c.get('label'):
                    continue

            json_data["controls"].append(c)

            if c.get('id'):
                ids.append(c.get('id'))

        self.req_id = mediaserver.send_message(json_data)
        self.whait_events()

    def whait_events(self):
        while True:
            data = mediaserver.whait_response(self.req_id)
            if data['action'] == 'close':
                return

            elif data['action'] == 'ok':
                if self.callback:
                    for mod in sys.modules.values():
                        if hasattr(mod, '__file__'):
                            f = os.path.splitext(mod.__file__)[0]
                            if f == os.path.splitext(self.module)[0]:
                                return_value = getattr(mod, self.callback)(self.item, dict([
                                    (
                                        base64.b64decode(c['id']),
                                        c['value']
                                    )
                                    for c in data['values'] if not c['type'] == 'label' and c.get('id')]))
                                return return_value
                else:
                    values = dict(
                        [(base64.b64decode(c['id']), c['value']) for c in data['values'] if
                         not c['type'] == 'label' and c.get('id')])
                    settings.set_settings(values, self.module)
                return

            elif data['action'] == 'custom_button':
                for mod in sys.modules.values():
                    if hasattr(mod, '__file__'):
                        f = os.path.splitext(mod.__file__)[0]

                        if f == os.path.splitext(self.module)[0]:
                            self.custom_button['values'] = dict([
                                (
                                    base64.b64decode(c['id']),
                                    c['value']
                                )
                                for c in data['values'] if not c['type'] == 'label' and c.get('id')
                            ])
                            custom_button = getattr(mod, self.custom_button['callback'])(self.item, self.custom_button)
                            json_data = {
                                'action': 'SettingsWindow.custom_buttom',
                                'dialog_id': self.req_id,
                                'custom_button': custom_button
                            }
                            mediaserver.send_message(json_data)

    @staticmethod
    def color_translate(m):
        ma = re.compile(r"((?:0x)?[0-f]{2})([0-f]{2})([0-f]{2})([0-f]{2})").match(m)
        if ma:
            return "rgba(%s,%s,%s,%s)" % (
                int(ma.group(2), 16),
                int(ma.group(3), 16),
                int(ma.group(4), 16),
                int(ma.group(1), 16) / 255.0
            )
        ma = re.compile(r"((?:0x)?[0-f]{2})([0-f]{2})([0-f]{2})").match(m)
        if ma:
            return "rgb(%s,%s,%s)" % (
                int(ma.group(1), 16),
                int(ma.group(2), 16),
                int(ma.group(3), 16),
            )

        return m
